﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SLinq;

public class Road : MonoBehaviour 
{
	public Vector3[] points;
	
	public static Road CreateFrom(Vector3[] points, Vector3[] roadLayout, Vector3 up, Material material)
	{
		GameObject roadGameObject = new GameObject("Road");
		
		points = points.Select(p => roadGameObject.transform.InverseTransformPoint(p)).ToArray();
		
		Road road = roadGameObject.AddComponent<Road>();
		road.points = points;
		
		MeshRenderer meshR = roadGameObject.AddComponent<MeshRenderer>();
		meshR.sharedMaterial = material;
		
		MeshFilter meshF = roadGameObject.AddComponent<MeshFilter>();
		meshF.mesh = CreateMesh(points, roadLayout, up);
		return road;
	}

	private static Mesh CreateMesh (Vector3[] points, Vector3[] roadLayout, Vector3 up)
	{
		int vertexCount = points.Length * roadLayout.Length;

		Vector3[] vertices = new Vector3[vertexCount];
		for (int i = 0; i < points.Length; i++) 
		{
			int prevIndex = ((i - 1) + points.Length) % points.Length,
				nextIndex = (i + 1) % points.Length;
			Vector3 forward = (points[nextIndex] - points[prevIndex]).normalized;
			Vector3 left = Vector3.Cross(forward, up).normalized;
			for (int j = 0; j < roadLayout.Length; j++) 
			{
				int index = i * roadLayout.Length + j;
				vertices[index] = points[i] + roadLayout[j].x * left + roadLayout[j].y * up + roadLayout[j].z * forward;
			}
		}
		
		List<int> triangles = new List<int>(points.Length * (roadLayout.Length - 1) * 6);
		for (int i = 0; i < points.Length; i++) 
		{
			for (int j = 1; j < roadLayout.Length; j++) 
			{
				int pi = ((i - 1) + points.Length) % points.Length;
				int p1 = pi * roadLayout.Length + (j - 1), 
					p2 = pi * roadLayout.Length + j, 
					c1 = i * roadLayout.Length + (j - 1), 
					c2 = i * roadLayout.Length + j;
				
				triangles.Add(p1);
				triangles.Add(p2);
				triangles.Add(c1);
				
				triangles.Add(c2);
				triangles.Add(c1);
				triangles.Add(p2);
			}
		}
		
		Mesh m = new Mesh();
		m.vertices = vertices;
		m.triangles = triangles.ToArray();
		m.RecalculateBounds();
		m.RecalculateNormals();
		return m;
	}
	
	protected void OnDrawGizmos()
	{
		Gizmos.matrix = transform.localToWorldMatrix;
		
		Gizmos.color = new Color(0f, 0f, 0f, 0.4f);
		for (int i = 1; i < points.Length; i++)
		{
			Gizmos.DrawSphere(points[i], 0.1f);
			Gizmos.DrawLine(points[i - 1], points[i]);
		}
		Gizmos.DrawLine(points[points.Length - 1], points[0]);
		
		Gizmos.color = Color.red;
		Gizmos.DrawSphere(points[0], 0.2f);
	}
}